/********************************************************************/
/*                                                                  */
/*  Licensed Materials - Property of IBM.                           */
/*  5724-T42                                                        */
/*  5724-T43                                                        */
/*  Copyright IBM Corp. 2008.                                       */
/*  All Rights Reserved.                                            */
/*  US Government Users Restricted Rights -                         */
/*  Use, duplication or disclosure restricted by                    */
/*  GSA ADP Schedule Contract with IBM Corp.                        */
/*                                                                  */
/*  This file contains sample code.  You may copy, modify, and      */
/*  distribute these samples, or their modifications, in any        */
/*  form, internally or as part of Your application or related      */
/*  documentation. These samples have not been tested under all     */
/*  conditions and are provided to You by IBM without obligation    */
/*  of support of any kind. IBM PROVIDES THESE SAMPLES "AS IS"      */
/*  SUBJECT TO ANY STATUTORY WARRANTIES THAT CANNOT BE EXCLUDED.    */
/*  IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS OR        */
/*  IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES   */
/*  OR CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR      */
/*  PURPOSE, AND NON-INFRINGEMENT REGARDING THESE SAMPLES OR        */
/*  TECHNICAL SUPPORT, IF ANY. You will indemnify IBM or third      */
/*  parties that provide IBM products ("Third Parties") from and    */
/*  against any third party claim arising out of the use,           */
/*  modification or distribution of these samples with Your         */
/*  application. You may not use the same path name as the          */
/*  original files/modules. You must not alter or delete any        */
/*  copyright information in the Samples.                           */
/*                                                                  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>

struct timeval timev1, timev2;
float elapsed_time;

extern double explicit_EO_call(double, double, double, double, double, double, int, int, double);
extern double explicit_American_put(double, double, double, double, double, double, int, int, double);

/***************************************************
 Explicit Finite difference scheme for computing 
 European CALL Option (with dividend) and American PUT Option
 Input Parameters:
 Strike Value K 
 Time T 
 Current Stock Price S 
 volality sigma sigma (in decimal - not in %)
 rate of return r (in decimal - not in %)
 dividend div (in decimal - not in %) 
 Number of Time Steps N_Time_Steps 
 Number of Stock Steps N_S_Divisions  
 dx 
***************************************************/

int main(int argc, char *argv[]) {

	double *Strike, *Expiration, *CurrentS;
	double *sigma, *rate, *dividend;
	int *Total_N, *Total_Nj;
	int i, j, k, l;
	int ncycles, cycle;
	double *dx;
	double *Payoff_EO_call, *Payoff_AM_put;

	if (argc !=2)
		ncycles = 1024;
	else
		ncycles = atoi(argv[1]);	

	Strike         = (double *) malloc(ncycles * sizeof(double));
	Expiration     = (double *) malloc(ncycles * sizeof(double)); 
	CurrentS       = (double *) malloc(ncycles * sizeof(double)); 
	sigma          = (double *) malloc(ncycles * sizeof(double)); 
	rate           = (double *) malloc(ncycles * sizeof(double)); 
	dividend       = (double *) malloc(ncycles * sizeof(double)); 
	Total_N        = (int *) malloc(ncycles * sizeof(int)); 
	Total_Nj       = (int *) malloc(ncycles * sizeof(int)); 
	dx             = (double *) malloc(ncycles * sizeof(double)); 
	Payoff_EO_call = (double *) malloc(ncycles * sizeof(double)); 
	Payoff_AM_put  = (double *) malloc(ncycles * sizeof(double)); 

	/* Hard coded input parameters */
	for (cycle = 0; cycle < ncycles; ++cycle) {
		Strike[cycle]     = 50.0;
		Expiration[cycle] = 1.0;
		CurrentS[cycle]   = 100.0;
		sigma[cycle]      = 0.40;
		rate[cycle]       = 0.10;
		dividend[cycle]   = 0.0;
		Total_N[cycle]    = 1000;
		Total_Nj[cycle]   = Total_N[cycle];
		dx[cycle]         = 0.20;
	}

	gettimeofday(&timev1, NULL);

	#pragma omp parallel for default(shared) private(cycle)
	for (cycle = 0; cycle < ncycles; ++cycle) {
		Payoff_EO_call[cycle] = explicit_EO_call(Strike[cycle],Expiration[cycle],
							CurrentS[cycle],sigma[cycle],
							rate[cycle],dividend[cycle],
							Total_N[cycle],Total_Nj[cycle],
							dx[cycle]);
		Payoff_AM_put[cycle] = explicit_American_put(Strike[cycle],Expiration[cycle],
							CurrentS[cycle],sigma[cycle],
							rate[cycle],dividend[cycle],
							Total_N[cycle],Total_Nj[cycle],
							dx[cycle]);
	}

	gettimeofday(&timev2, NULL);
	elapsed_time = (timev2.tv_sec - timev1.tv_sec) + 0.000001 * (timev2.tv_usec - timev1.tv_usec);

	int result_valid = 1;
	for (cycle = 0; cycle < ncycles; ++cycle) {
		if (abs(Payoff_EO_call[cycle]-55.06638) > 1E-5) {
			result_valid = 0;
		}
		if (abs(Payoff_AM_put[cycle]-10.346994) > 1E-5) {
			result_valid = 0;
		}
	}

	if (result_valid)
		printf("Result is Valid\n");
	else
		printf("Result is Invalid\n");

	printf("Elapsed time is %f seconds.\n", elapsed_time);

	return(0);
}
