/********************************************************************/
/*                                                                  */
/*  Licensed Materials - Property of IBM.                           */
/*  5724-T42                                                        */
/*  5724-T43                                                        */
/*  Copyright IBM Corp. 2008.                                       */
/*  All Rights Reserved.                                            */
/*  US Government Users Restricted Rights -                         */
/*  Use, duplication or disclosure restricted by                    */
/*  GSA ADP Schedule Contract with IBM Corp.                        */
/*                                                                  */
/*  This file contains sample code.  You may copy, modify, and      */
/*  distribute these samples, or their modifications, in any        */
/*  form, internally or as part of Your application or related      */
/*  documentation. These samples have not been tested under all     */
/*  conditions and are provided to You by IBM without obligation    */
/*  of support of any kind. IBM PROVIDES THESE SAMPLES "AS IS"      */
/*  SUBJECT TO ANY STATUTORY WARRANTIES THAT CANNOT BE EXCLUDED.    */
/*  IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS OR        */
/*  IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES   */
/*  OR CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR      */
/*  PURPOSE, AND NON-INFRINGEMENT REGARDING THESE SAMPLES OR        */
/*  TECHNICAL SUPPORT, IF ANY. You will indemnify IBM or third      */
/*  parties that provide IBM products ("Third Parties") from and    */
/*  against any third party claim arising out of the use,           */
/*  modification or distribution of these samples with Your         */
/*  application. You may not use the same path name as the          */
/*  original files/modules. You must not alter or delete any        */
/*  copyright information in the Samples.                           */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/*                                                                  */
/* DESCRIPTION:                                                     */
/*    The sample shows a simple OpenMP program that calls a SPU     */
/*    user-defined function in a parallel region to add two 2D      */
/*    arrays. The user-defined function uses __ea pointers to point */
/*    to the data in PPU memory. The rows are distributed amongst   */
/*    the SPUs using OpenMP directives, and the columns are added   */
/*    using the SPU instrinsic spu_add.                             */
/*                                                                  */
/********************************************************************/

#include <stdio.h>

#define BLOCK_SIZE 64 
#define BLOCK_NUM 16

int data1[BLOCK_NUM][BLOCK_SIZE];
int data2[BLOCK_NUM][BLOCK_SIZE];
int result[BLOCK_NUM][BLOCK_SIZE];

/* Prototype of user-defined SPU function */
#ifdef __cplusplus
extern "C"
#endif
void spu_sum_array(int *data1, int *data2, int *result, int size);

int main() {
	int i,j;
	int row_sum[BLOCK_SIZE];

	/* Initialize the 2D array */
	for (i = 0; i < BLOCK_NUM; i++) {
		for (j = 0; j < BLOCK_SIZE; j++) {
			data1[i][j] = i+j;
			data2[i][j] = 2*i+j;
		}
	}

	/* Compute the sum of the arrays using the SPUs */
	#pragma omp parallel for
	for (i = 0; i < BLOCK_NUM; i++) {
		spu_sum_array(data1[i], data2[i], result[i], BLOCK_SIZE);
	}

	/* Print out the result */
	for (i = 0; i < BLOCK_NUM; i++) {
		for (j = 0; j < BLOCK_SIZE; j++) {
			printf("%4d", result[i][j]);
		}
		printf("\n");
	}

	return 0;
}
